/*
 * jBoss, the OpenSource EJB server
 *
 * Distributable under GPL license.
 * See terms of license at gnu.org.
 */
 
package org.jboss.security;


import java.util.Map;
import javax.security.auth.Subject;
import javax.security.auth.callback.Callback;
import javax.security.auth.callback.CallbackHandler;
import javax.security.auth.callback.NameCallback;
import javax.security.auth.callback.PasswordCallback;
import javax.security.auth.callback.UnsupportedCallbackException;
import javax.security.auth.login.LoginException;
import javax.security.auth.spi.LoginModule;
import org.jboss.system.SecurityAssociation;
import org.jboss.system.SimplePrincipal;


public class ClientLoginModule implements LoginModule {
    private CallbackHandler _callbackHandler;

    /**
     * Initialize this LoginModule.
     */
    public void initialize(Subject subject, CallbackHandler callbackHandler,
            Map sharedState, Map options) {
        _callbackHandler = callbackHandler;
    }

    /**
     * Method to authenticate a Subject (phase 1).
     */
    public boolean login() throws LoginException {
        Callback[] callbacks = new Callback[2];
        
        // prompt for a username and password
        if (_callbackHandler == null)
            throw new LoginException("Error: no CallbackHandler available " +
                "to garner authentication information from the user");

        callbacks[0] = new NameCallback("User name: ", "guest");
        callbacks[1] = new PasswordCallback("Password: ", false);
        try {
            String username;
            char[] password = null;
            char[] tmpPassword;

            _callbackHandler.handle(callbacks);
            username = ((NameCallback)callbacks[0]).getName();
            SecurityAssociation.setPrincipal(new SimplePrincipal(username));
            tmpPassword = ((PasswordCallback)callbacks[1]).getPassword();
            if (tmpPassword != null) {
                password = new char[tmpPassword.length];
                System.arraycopy(tmpPassword, 0, password, 0, tmpPassword.length);
                ((PasswordCallback)callbacks[1]).clearPassword();
            }
            SecurityAssociation.setCredential(password);
        } catch (java.io.IOException ioe) {
            throw new LoginException(ioe.toString());
        } catch (UnsupportedCallbackException uce) {
            throw new LoginException("Error: " + uce.getCallback().toString() +
                    " not available to garner authentication information " +
                    "from the user");
        }
        return true;
    }
          
    /**
     * Method to commit the authentication process (phase 2).
     */
    public boolean commit() throws LoginException {
        return true;
    }    
          
    /**
     * Method to abort the authentication process (phase 2).
     */
    public boolean abort() throws LoginException {
        SecurityAssociation.setPrincipal(null);
        SecurityAssociation.setCredential(null);
        return true;
    }

    public boolean logout() throws LoginException {
        return true;
    }
}
