package ckorn.studienarbeit.client;

import javax.naming.*;
import ckorn.studienarbeit.bibliothek.interfaces.*;
import ckorn.studienarbeit.buch.interfaces.*;
import java.util.*;
import javax.ejb.*;
import javax.rmi.*; 
import java.rmi.*;

/** Einfacher Client um die Bibliotheks-Bean zu testen */

public class BibliotheksClient {

	public static void main(String[] args) {

		// Umgebungsvariablen fr JNDI-Service setzen
		System.setProperty("java.naming.factory.initial", "org.jnp.interfaces.NamingContextFactory");
		System.setProperty("java.naming.provider.url", "localhost:1099");
		
		try {
			// Initalen Kontext setzen
			InitialContext context = new InitialContext();
			
			// Referenz auf Home-Objekt der Bibliotheks-Bean holen
			Object ref  = context.lookup("bibliothek/Bibliothek");
			BibliothekHome bibhome = (BibliothekHome) PortableRemoteObject.narrow (ref, BibliothekHome.class);

			// Referenz auf Remote-Objekt der Bibliotheks.Bean holen
			BibliothekRemote bibliothek = bibhome.create();

			// Einige Buecher in die Bibliothek einfgen
			bibliothek.addBook(1, "Sun Microsystems", "Enterprise JavaBeans Specification 1.1", "Sun Microsystems Inc.");
			System.out.println("Buch hinzugefgt: ");
			System.out.println(bibliothek.getBookByInvNo(1).getStringRepresentation());

			bibliothek.addBook(2, "Ed Roman", "Mastering Enterprise JavaBeans", "Wiley");
			System.out.println("Buch hinzugefgt: ");
			System.out.println(bibliothek.getBookByInvNo(2).getStringRepresentation());

			bibliothek.addBook(3, "Stefan Danninger, Ingo Peters", "Enterprise JavaBeans", "Addison-Wesley");
			System.out.println("Buch hinzugefgt: ");
			System.out.println(bibliothek.getBookByInvNo(3).getStringRepresentation());

			bibliothek.addBook(42, "Douglas Adams", "The hitch hikers guide to the galaxy", "Heinemann");
			System.out.println("Buch hinzugefgt: ");
			System.out.println(bibliothek.getBookByInvNo(42).getStringRepresentation());

			bibliothek.addBook(4, "Douglas Adams", "The restaurant at the end of the universe", "Heinemann");
			System.out.println("Buch hinzugefgt: ");
			System.out.println(bibliothek.getBookByInvNo(4).getStringRepresentation());

			// Anzahl der Bcher ausgeben
			System.out.println("Anzahl der Bcher: "+bibliothek.countBooks()+"\n");

			// Alle Bcher von Douglas Adams suchen und ausgeben
			System.out.println("Alle Bcher von Douglas Adams:");
			Collection someBooks = bibliothek.getAuthorsBooks("Douglas Adams");
			Iterator someBookIterator = someBooks.iterator();
			while (someBookIterator.hasNext()) {
				System.out.println(((BuchRemote) someBookIterator.next()).getStringRepresentation());
			}		


			// Alle Bcher wieder lschen
			bibliothek.removeBook(1);
			System.out.println("Buch mit Inventarnummer 1 gelscht");
			bibliothek.removeBook(2);
			System.out.println("Buch mit Inventarnummer 2 gelscht");
			bibliothek.removeBook(3);
			System.out.println("Buch mit Inventarnummer 3 gelscht");
			bibliothek.removeBook(4);
			System.out.println("Buch mit Inventarnummer 3 gelscht");
			bibliothek.removeBook(42);
			System.out.println("Buch mit Inventarnummer 42 gelscht");

		}
		catch(RemoteException e) {
      	System.out.println("Fehler: Es konnte keine Verbingung per RMI zu der Bean aufgebaut werden!");
		}
		catch(FinderException e) {
      	System.out.println("Fehler: Es wurde keine Bean mit dieser Inventarnummer gefunden!");
		}
		catch(CreateException e) {
      	System.out.println("Fehler: Zugriff auf Remote-Objekt der Bibliotheks-Bean fehlgeschlagen!");
		}
		catch(RemoveException e) {
      	System.out.println("Fehler: Bean konnte nicht gelscht werden!");
		}		
		catch(NamingException e) {
      	System.out.println("Fehler: Bean konnte nicht per JNDI gefunden werden!");
		}		
	}
}
